# WARNING - Generated by {fusen} from /dev/flat_full.Rmd: do not edit by hand




test_that("Test is_mixedEffect_inFormula", {
  formula1 <- y ~ a + (1 | B)
  formula2 <- ~ a + (1 | B)
  formula3 <- x ~ c + d

  expect_true(is_mixedEffect_inFormula(formula1))
  expect_true(is_mixedEffect_inFormula(formula2))
  expect_false(is_mixedEffect_inFormula(formula3))
})

test_that("Test is_formula_mixedTypeI", {
  formula1 <- y ~ x + (1 | group)
  formula2 <- y ~ z + group1 + (1 | group1)
  formula3 <- y ~ z + (1 | group1 + group2)
  formula4 <- y ~ z + (1 | group1/z)
  formula5 <- y ~ z + ( group | z ) ## z is fixed then expected on the left in parenthesis
  
  expect_true(is_formula_mixedTypeI(formula1))
  expect_false(is_formula_mixedTypeI(formula2))
  expect_false(is_formula_mixedTypeI(formula3))
  expect_false(is_formula_mixedTypeI(formula4))
  expect_false(is_formula_mixedTypeI(formula5))


})


test_that("getCategoricalVar_inFixedEffect returns the correct result", {
  
    ###### PREPARE DATA
    N_GENES = 2
    MAX_REPLICATES = 4
    MIN_REPLICATES = 4

    input_var_list <- init_variable( name = "genotype", mu = 2, sd = 0.5, level = 10) %>%
      init_variable( name = "environment", mu = c(1, 3), sd = NA, level = 2) %>%
      add_interaction(between_var = c("genotype", 'environment'), mu = 1, sd = 0.39)
    
    mock_data <- mock_rnaseq(input_var_list, N_GENES,
                             min_replicates = MIN_REPLICATES,
                             max_replicates = MAX_REPLICATES,
                             basal_expression = 3, dispersion = 100)
    
    data2fit = prepareData2fit(countMatrix = mock_data$counts, metadata =  mock_data$metadata, normalization = F)
    
    l_tmb <- fitModelParallel(formula = kij ~  environment  + (environment | genotype ),
                              data = data2fit, group_by = "geneID",
                              family = glmmTMB::nbinom2(link = "log"), n.cores = 1)
      
  
    tidy_tmb <- tidy_tmb(l_tmb)
    categorical_var <- getCategoricalVar_inFixedEffect(tidy_tmb)
    expect_equal(categorical_var, "label_environment")
})

test_that("group_logQij_per_genes_and_labels returns the correct result", {
    
    ############ PREPARE DATA
    N_GENES = 2
    MAX_REPLICATES = 4
    MIN_REPLICATES = 4
    input_var_list <- init_variable( name = "genotype", mu = 2, sd = 0.5, level = 10) %>%
      init_variable( name = "environment", mu = c(1, 3), sd = NA, level = 2) %>%
      add_interaction(between_var = c("genotype", 'environment'), mu = 1, sd = 0.39)
    
    mock_data <- mock_rnaseq(input_var_list, N_GENES,
                             min_replicates = MIN_REPLICATES,
                             max_replicates = MAX_REPLICATES,
                             basal_expression = 3, dispersion = 100)
    
    data2fit = prepareData2fit(countMatrix = mock_data$counts, metadata =  mock_data$metadata, normalization = F)
    
    l_tmb <- fitModelParallel(formula = kij ~  environment  + (environment | genotype ),
                              data = data2fit, group_by = "geneID",
                              family = glmmTMB::nbinom2(link = "log"), n.cores = 1)
    
    ground_truth_eff <- mock_data$groundTruth$effects
    categorical_var <- "label_environment"
    logqij_list <- group_logQij_per_genes_and_labels(ground_truth_eff, categorical_var)
    
    expect_is(logqij_list, "data.frame")
    expect_equal(attributes(logqij_list)$names , c("gene1", "gene2"))
    expect_equal(length(logqij_list$gene1), 2)
    expect_equal(length(logqij_list$gene2), 2)
    expect_equal(length(logqij_list$gene2[[1]]), 10)
})

test_that("getActualMixed_typeI returns the correct result", {
   ############ PREPARE DATA
    N_GENES = 2
    MAX_REPLICATES = 4
    MIN_REPLICATES = 4
    input_var_list <- init_variable( name = "genotype", mu = 2, sd = 0.5, level = 10) %>%
      init_variable( name = "environment", mu = c(1, 3), sd = NA, level = 2) %>%
      add_interaction(between_var = c("genotype", 'environment'), mu = 1, sd = 0.39)
    
    mock_data <- mock_rnaseq(input_var_list, N_GENES,
                             min_replicates = MIN_REPLICATES,
                             max_replicates = MAX_REPLICATES,
                             basal_expression = 3, dispersion = 100)
    
    data2fit = prepareData2fit(countMatrix = mock_data$counts, metadata =  mock_data$metadata, normalization = F)
    
    l_tmb <- fitModelParallel(formula = kij ~  environment  + (environment | genotype ),
                              data = data2fit, group_by = "geneID",
                              family = glmmTMB::nbinom2(link = "log"), n.cores = 1)
    
    ground_truth_eff <- mock_data$groundTruth$effects
    categorical_var <- "label_environment"
    logqij_list <- group_logQij_per_genes_and_labels(ground_truth_eff, categorical_var)
    l_genes <- unique(ground_truth_eff$geneID)
    genes_iter_list <- stats::setNames(l_genes, l_genes)
    categoricalVar_infos= list(ref = "environment1", 
                             labels = c("environment1", "environment2"), 
                             labelsOther = "environment2")
    
    ## -- test
    actual_mixedEff <- getActualMixed_typeI(logqij_list, 
                                              genes_iter_list, 
                                                categoricalVar_infos)
    
    ## -- verif
    expect_is(actual_mixedEff, "data.frame")
    expect_equal(colnames(actual_mixedEff), c("actual", "term", "description", "geneID"))
    expect_equal(unique(actual_mixedEff$geneID), c("gene1", "gene2"))
    expect_equal(unique(actual_mixedEff$term), c("(Intercept)", "environment2", 
                                                 "sd_(Intercept)", "sd_environment2", "cor__(Intercept).environment2"))

})


# Test for InferenceToExpected_withMixedEff
test_that("inferenceToExpected_withMixedEff correctly compares inference to expected values", {
  
  ## -- PREPARE DATA
  N_GENES = 2
  MAX_REPLICATES = 4
  MIN_REPLICATES = 4
  
  input_var_list <- init_variable(name = "genotype", mu = 2, sd = 0.5, level = 10) %>%
  init_variable(name = "environment", mu = c(1, 3), sd = NA, level = 2) %>%
  add_interaction(between_var = c("genotype", 'environment'), mu = 1, sd = 0.39)
  
  mock_data <- mock_rnaseq(input_var_list, N_GENES,
                         min_replicates = MIN_REPLICATES,
                         max_replicates = MAX_REPLICATES,
                         basal_expression = 3, dispersion = 100)
  
  data2fit <- prepareData2fit(countMatrix = mock_data$counts, metadata = mock_data$metadata, normalization = FALSE)
  
  l_tmb <- fitModelParallel(formula = kij ~ environment + (environment | genotype),
                          data = data2fit, group_by = "geneID",
                          family = glmmTMB::nbinom2(link = "log"), n.cores = 1)

  ## -- call fonction to test
  compared_df <- inferenceToExpected_withMixedEff(tidy_tmb(l_tmb), mock_data$groundTruth$effects)
  
  ## -- TEST VERIF
  expect_equal(c("term", "description", "geneID", "effect", 
                "component", "group", "estimate", "std.error", 
                "statistic", "p.value", "actual" ) , colnames(compared_df))
  expect_equal(c("gene1", "gene2" ) , unique(compared_df$geneID))
  expect_equal(unique(compared_df$term), c("(Intercept)", "cor__(Intercept).environment2", "environment2", 
                                                 "sd_(Intercept)", "sd_environment2"))

})

# Test for calculate_actualMixed
test_that("calculate_actualMixed calculates actual mixed effects as expected", {
   ## -- PREPARE DATA
  N_GENES = 2
  MAX_REPLICATES = 4
  MIN_REPLICATES = 4
  
  input_var_list <- init_variable(name = "genotype", mu = 2, sd = 0.5, level = 10) %>%
  init_variable(name = "environment", mu = c(1, 3), sd = NA, level = 2) %>%
  add_interaction(between_var = c("genotype", 'environment'), mu = 1, sd = 0.39)
  
  mock_data <- mock_rnaseq(input_var_list, N_GENES,
                         min_replicates = MIN_REPLICATES,
                         max_replicates = MAX_REPLICATES,
                         basal_expression = 3, dispersion = 100)
  
  data2fit <- prepareData2fit(countMatrix = mock_data$counts, metadata = mock_data$metadata, normalization = FALSE)
  
  
  ground_truth_eff <- mock_data$groundTruth$effects
  categorical_var <- "label_environment"
  logqij_list <- group_logQij_per_genes_and_labels(ground_truth_eff, categorical_var)
  l_genes <- unique(ground_truth_eff$geneID)
  genes_iter_list <- stats::setNames(l_genes, l_genes)
  categoricalVar_infos= list(ref = "environment1", 
                           labels = c("environment1", "environment2"), 
                           labelsOther = "environment2")
    
  ## -- call function & test
  data_per_gene <- lapply(genes_iter_list, function(g) {
                          data_gene <- data.frame(logqij_list[[g]])
                          colnames(data_gene) <- categoricalVar_infos$labels
                          return(data_gene)
                    })
  data_gene <- data_per_gene$gene1
  actual_mixed <- calculate_actualMixed(data_gene, 
                                        labelRef_InCategoricalVar = categoricalVar_infos$ref ,
                                        labelOther_inCategoricalVar = categoricalVar_infos$labelsOther)
  expect_equal( colnames(actual_mixed), c("actual", "term", "description"))
  expect_equal(actual_mixed$term, c("(Intercept)", "environment2", 
                                    "sd_(Intercept)", "sd_environment2", 
                                    "cor__(Intercept).environment2"))
  expect_equal(actual_mixed$description, c("(Intercept)", "environment", 
                                    "sd", "sd", 
                                    "cor"))
})



