# WARNING - Generated by {fusen} from /dev/flat_full.Rmd: do not edit by hand


# Test unitaires
test_that("wald_test performs correct tests", {
  # Test with "greater" alternative
  result_greater <- wald_test(estimation = 0.1, std_error = 0.02, reference_value = 0.05, alternative = "greater")
  expect_equal(result_greater$p.value, 1 - pnorm((0.1 - 0.05) / 0.02, mean = 0, sd = 1, lower.tail = TRUE))

  # Test with "less" alternative
  result_less <- wald_test(estimation = 0.1, std_error = 0.02, reference_value = 0.05, alternative = "less")
  expect_equal(result_less$p.value, pnorm((0.1 - 0.05) / 0.02, mean = 0, sd = 1, lower.tail = TRUE))

  # Test with "greaterAbs" alternative
  result_greaterAbs <- wald_test(estimation = 0.1, std_error = 0.02, reference_value = 0.05, alternative = "greaterAbs")
  expect_equal(result_greaterAbs$p.value, (2 * (1 - pnorm((abs(0.1) - 0.05) / 0.02, mean = 0, sd = 1, lower.tail = TRUE))))

  # Test with invalid alternative
  expect_error(wald_test(estimation = 0.1, std_error = 0.02, reference_value = 0.05, alternative = "invalid"))
})



test_that("results function performs statistical tests correctly", {
  # Charger les données iris pour les tests
  data(iris)
  # Fit models and perform statistical tests
  model_list <- fitModelParallel(formula = Sepal.Length ~ Sepal.Width + Petal.Length, 
                                 data = iris, group_by = "Species", n.cores = 1) 
  results_df <- tidy_results(model_list, coeff_threshold = 0.1, alternative_hypothesis = "greater")

  # Vérifier que les colonnes 'statistic' et 'p.value' ont été ajoutées au dataframe
  expect_true("statistic" %in% colnames(results_df))
  expect_true("p.value" %in% colnames(results_df))

  # Vérifier que les tests statistiques ont été effectués correctement
  # Ici, nous ne vérifierons pas les valeurs exactes des résultats car elles peuvent varier en fonction de la machine et des packages utilisés.
  # Nous nous assurerons seulement que les résultats sont dans le format attendu.
  expect_is(results_df$statistic, "numeric")
  expect_is(results_df$p.value, "numeric")
  expect_is(results_df$p.adj, "numeric")


  # Vérifier que les p-values ne dépassent pas 1
  expect_true(all(results_df$p.value <= 1))

  # Vérifier que les valeurs sont correctes pour les colonnes 'statistic' et 'p.value'
  # (Cela dépend des données iris et des modèles ajustés)
  # Remarque : Vous devrez peut-être ajuster ces tests en fonction des valeurs réelles des données iris et des modèles ajustés.
  expect_true(all(!is.na(results_df$statistic)))
  expect_true(all(!is.na(results_df$p.value)))

  # Vérifier que le seuil des coefficients et l'hypothèse alternative sont correctement appliqués
  # Ici, nous nous attendons à ce que les p-values soient uniquement pour les coefficients dépassant le seuil
  expect_true(all(ifelse(abs(results_df$estimate) > 0.1, results_df$p.value <= 1, results_df$p.value == 1)))
  expect_true(all(ifelse(abs(results_df$estimate) > 0.1, results_df$p.adj <= 1, results_df$p.adj == 1)))

  })




